import Binance from "node-binance-api";
import dotenv from "dotenv";
import fetch from "node-fetch";
// import { sendWhatsAppMessage } from "./whatsapp.js";

dotenv.config();

let previousTotalInIDR = 0;

const binance = new Binance().options({
  APIKEY: process.env.BINANCE_API_KEY,
  APISECRET: process.env.BINANCE_API_SECRET,
  useServerTime: true,
});

async function getUSDTtoIDR() {
  try {
    const response = await fetch(
      "https://api.coingecko.com/api/v3/simple/price?ids=tether&vs_currencies=idr"
    );
    const data = await response.json();
    return data.tether.idr;
  } catch (err) {
    console.error("Gagal mengambil harga USDT ke IDR, fallback ke 16800");
    return 16800;
  }
}

export async function checkSpotBalanceInIDR(isTrigger) {
  try {
    const balances = await binance.balance();
    const prices = await binance.prices();
    // const usdtToIdr = await getUSDTtoIDR();

    console.clear();
    console.log(
      `📊 Saldo SPOT dalam IDR (USDT ≈ Rp ${usdtToIdr.toLocaleString("id-ID")}):`
    );

    let totalInIDR = 0;
    let summary = `📊 *Saldo SPOT Binance*\n(USDT ≈ Rp ${usdtToIdr.toLocaleString("id-ID")})\n\n`;

    for (const [asset, info] of Object.entries(balances)) {
      const available = parseFloat(info.available);
      if (available <= 0) continue;

      let priceInUSDT = 0;
      if (asset === "USDT") {
        priceInUSDT = 1;
      } else if (prices[`${asset}USDT`]) {
        priceInUSDT = parseFloat(prices[`${asset}USDT`]);
      } else if (prices[`${asset}BUSD`]) {
        priceInUSDT = parseFloat(prices[`${asset}BUSD`]);
      } else {
        continue;
      }

      const valueInIDR = available * priceInUSDT * usdtToIdr;
      if (valueInIDR < 20000) continue;

      totalInIDR += valueInIDR;
      summary += `• ${asset}: ${available} ≈ Rp ${valueInIDR.toLocaleString("id-ID")}\n`;
    }

    summary += `\n💰 *Total: Rp ${totalInIDR.toLocaleString("id-ID")}*\n🕒 ${new Date().toLocaleString("id-ID")}`;

    console.log(summary);

    // if(isTrigger) {
    //   const waNumber = process.env.WA_TARGET;
    //   await sendWhatsAppMessage(waNumber, summary);
    //   console.log("✅ Notifikasi saldo terkirim!");
    //   return true;
    // }


    const previousRounded = Math.floor(previousTotalInIDR / 100000) * 100000;
    const currentRounded = Math.floor(totalInIDR / 100000) * 100000;

    if (previousRounded !== currentRounded && previousTotalInIDR !== 0) {
      const waNumber = process.env.WA_TARGET;
      const notificationMessage = `⚠️ Perubahan saldo terdeteksi!\nSaldo sebelumnya: Rp ${previousTotalInIDR.toLocaleString(
        "id-ID"
      )}\nSaldo sekarang: Rp ${totalInIDR.toLocaleString("id-ID")}`;
      // await sendWhatsAppMessage(waNumber, notificationMessage);
      console.log("✅ Notifikasi perubahan saldo terkirim!");
    }

    // Perbarui saldo sebelumnya
    previousTotalInIDR = totalInIDR;
    return true;
  } catch (err) {
    console.error("Terjadi kesalahan:", err.message || err);
  }
}


export async function sellSelectedAssets(symbolName, symbolPair,phoneNumber) {
  // Fungsi untuk menjual aset di spot berdasarkan aset yang dipilih
  try {
    const getSpotBalance = await binance.balance();
    const balanceAmount = getSpotBalance[symbolName].available;
    const prices = await binance.prices();
    const priceInUSDT = parseFloat(prices[`${symbolName}${symbolPair}`]);
    const usdtToIdr = await getUSDTtoIDR();
    const valueInIDR = balanceAmount * priceInUSDT * usdtToIdr;
    const waNumber = phoneNumber;
    const notificationMessage = `🔄 Menjual ${symbolName} sebanyak ${balanceAmount} dengan nilai Rp ${valueInIDR.toLocaleString(
      "id-ID"
    )}`;
    // await sendWhatsAppMessage(waNumber, notificationMessage);
    console.log("✅ Notifikasi penjualan terkirim!");
    console.log(
      `Menjual ${balanceAmount} ${symbolName} dengan nilai Rp ${valueInIDR.toLocaleString(
        "id-ID"
      )}`
    );
    try {
      await binance.marketSell(`${symbolName}${symbolPair}`, balanceAmount);
    } catch (error) {
      const errorMessage = `❌ Gagal menjual ${symbolName} sebanyak ${balanceAmount}.\n\nError: ${error.msg || error}`;
      // await sendWhatsAppMessage(waNumber, errorMessage);
      console.log(errorMessage);
      return;      
    }
    console.log(
      `✅ ${balanceAmount} ${symbolName} berhasil dijual!`
    );
    //set timer to wait for 10 seconds
    // await new Promise((resolve) => setTimeout(resolve, 10000));
    // Update the message to indicate completion
    const completedMessage = `✅ Penjualan ${symbolName} sebanyak ${balanceAmount} berhasil!`;
    // await sendWhatsAppMessage(waNumber, completedMessage);
    console.log(completedMessage);
  } catch (error) {
    console.error("Terjadi kesalahan saat menjual aset:", error.message || error);
  }
}

export async function getPrice(symbolName,symbolPair,phoneNumber) {
  // Fungsi untuk mendapatkan harga aset di spot berdasarkan aset yang dipilih
  try {
    const prices = await binance.prices();
    const priceInUSDT = parseFloat(prices[`${symbolName}${symbolPair}`]);
    if (!priceInUSDT) {
      console.log("Harga tidak ditemukan untuk simbol tersebut.");
      // await sendWhatsAppMessage(
      //   phoneNumber,
      //   "Harga tidak ditemukan untuk simbol tersebut."
      // );
      return;
    }
    const usdtToIdr = await getUSDTtoIDR();
    const valueInIDR = priceInUSDT * usdtToIdr;
    const waNumber = phoneNumber;
    const notificationMessage = `💰 Harga ${symbolName} saat ini adalah Rp ${valueInIDR.toLocaleString(
      "id-ID"
    )} atau ${priceInUSDT} USDT`;
    // await sendWhatsAppMessage(waNumber, notificationMessage);
    console.log("✅ Notifikasi harga terkirim!");
  } catch (error) {
    console.error("Terjadi kesalahan saat mendapatkan harga:", error.message || error);
  }
}

setInterval(() => {
  checkSpotBalanceInIDR();
}
, 1500000);

